@@ -61,7 +61,7 @@
 #include <sys/filio.h>  /* FIONREAD on Solaris 7 */
 #endif
 
-#include <setjmp.h>
+#include <setjmp.h>  /* for sigjmp_buf, sigsetjmp() */
 
 #ifdef HAVE_PWD_H
 #include <pwd.h>
@@ -70,8 +70,8 @@
 #include <ctype.h>
 
 /*****************************************************************************
-*  This is a rewrite/clone of the arpa/tftp.h file for systems without it.   *
-*****************************************************************************/
+ *  This is a rewrite/clone of the arpa/tftp.h file for systems without it.  *
+ *****************************************************************************/
 #define SEGSIZE 512 /* data segment size */
 
 #if defined(__GNUC__) && ((__GNUC__ >= 3) || \
@@ -105,12 +105,10 @@ struct tftphdr {
 #define TFTP_ENOUSER   7
 
 /*****************************************************************************
-*                      STRUCT DECLARATIONS AND DEFINES                       *
-*****************************************************************************/
+ *                      STRUCT DECLARATIONS AND DEFINES                      *
+ *****************************************************************************/
 
-#ifndef PKTSIZE
 #define PKTSIZE (SEGSIZE + 4)  /* SEGSIZE defined in arpa/tftp.h */
-#endif
 
 struct testcase {
   char *buffer;   /* holds the file data to send to the client */
@@ -157,13 +155,13 @@ struct bf {
 #define opcode_ACK   4
 #define opcode_ERROR 5
 
-#define TIMEOUT      5
+#define TIMEOUT 5
 
-#define REQUEST_DUMP  "server.input"
+#define REQUEST_DUMP "server.input"
 
 /*****************************************************************************
-*                              GLOBAL VARIABLES                              *
-*****************************************************************************/
+ *                              GLOBAL VARIABLES                             *
+ *****************************************************************************/
 
 static struct errmsg errmsgs[] = {
   { TFTP_EUNDEF,       "Undefined error code" },
@@ -207,52 +205,12 @@ static int tftpd_wroteportfile = 0;
 static sigjmp_buf timeoutbuf;
 #endif
 
+/*****************************************************************************
+ *                          FUNCTION IMPLEMENTATIONS                         *
+ *****************************************************************************/
+
 #if defined(HAVE_ALARM) && defined(SIGALRM)
 static const unsigned int rexmtval = TIMEOUT;
-#endif
-
-/*****************************************************************************
-*                            FUNCTION PROTOTYPES                             *
-*****************************************************************************/
-
-static struct tftphdr *rw_init(int);
-
-static struct tftphdr *w_init(void);
-
-static struct tftphdr *r_init(void);
-
-static void read_ahead(struct testcase *test, int convert);
-
-static ssize_t write_behind(struct testcase *test, int convert);
-
-static int synchnet(curl_socket_t);
-
-static int do_tftp(struct testcase *test, struct tftphdr *tp, ssize_t size);
-
-static int validate_access(struct testcase *test,
-                           const char *filename, unsigned short mode);
-
-static void sendtftp(struct testcase *test, const struct formats *pf);
-
-static void recvtftp(struct testcase *test, const struct formats *pf);
-
-static void nak(int error);
-
-#if defined(HAVE_ALARM) && defined(SIGALRM)
-
-static void mysignal(int sig, void (*handler)(int));
-
-static void timer(int signum);
-
-static void justtimeout(int signum);
-
-#endif /* HAVE_ALARM && SIGALRM */
-
-/*****************************************************************************
-*                          FUNCTION IMPLEMENTATIONS                          *
-*****************************************************************************/
-
-#if defined(HAVE_ALARM) && defined(SIGALRM)
 
 /*
  * Like signal(), but with well-defined semantics.
@@ -302,6 +260,36 @@ static void justtimeout(int signum)
 
 #endif /* HAVE_ALARM && SIGALRM */
 
+/*
+ * Send a nak packet (error message).  Error code passed in is one of the
+ * standard TFTP codes, or a Unix errno offset by 100.
+ */
+static void nak(int error)
+{
+  struct tftphdr *tp;
+  int length;
+  struct errmsg *pe;
+
+  tp = &trsbuf.hdr;
+  tp->th_opcode = htons(opcode_ERROR);
+  tp->th_code = htons((unsigned short)error);
+  for(pe = errmsgs; pe->e_code >= 0; pe++)
+    if(pe->e_code == error)
+      break;
+  if(pe->e_code < 0) {
+    curlx_strerror(error - 100, pe->e_msg, sizeof(pe->e_msg));
+    tp->th_code = TFTP_EUNDEF;   /* set 'undef' errorcode */
+  }
+  length = (int)strlen(pe->e_msg);
+
+  /* we use memcpy() instead of strcpy() in order to avoid buffer overflow
+   * report from glibc with FORTIFY_SOURCE */
+  memcpy(tp->th_msg, pe->e_msg, length + 1);
+  length += 5;
+  if(swrite(peer, &trsbuf.storage[0], length) != length)
+    logmsg("nak: fail\n");
+}
+
 /*
  * init for either read-ahead or write-behind.
  * zero for write-behind, one for read-head.
@@ -327,25 +315,6 @@ static struct tftphdr *r_init(void)
   return rw_init(1); /* read-ahead */
 }
 
-/* Have emptied current buffer by sending to net and getting ack.
-   Free it and return next buffer filled with data.
- */
-static int readit(struct testcase *test, struct tftphdr * volatile *dpp,
-                  int convert /* if true, convert to ASCII */)
-{
-  struct bf *b;
-
-  bfs[current].counter = BF_FREE; /* free old one */
-  current = !current;             /* "incr" current */
-
-  b = &bfs[current];              /* look at new buffer */
-  if(b->counter == BF_FREE)       /* if empty */
-    read_ahead(test, convert);    /* fill it */
-
-  *dpp = &b->buf.hdr;             /* set caller's ptr */
-  return b->counter;
-}
-
 /*
  * fill the input buffer, doing ASCII conversions if requested
  * conversions are  lf -> cr, lf  and cr -> cr, nul
@@ -407,19 +376,23 @@ static void read_ahead(struct testcase *test,
   b->counter = (int)(p - dp->th_data);
 }
 
-/* Update count associated with the buffer, get new buffer from the queue.
-   Calls write_behind only if next buffer not available.
+/* Have emptied current buffer by sending to net and getting ack.
+   Free it and return next buffer filled with data.
  */
-static int writeit(struct testcase *test, struct tftphdr * volatile *dpp,
-                   int ct, int convert)
+static int readit(struct testcase *test, struct tftphdr * volatile *dpp,
+                  int convert /* if true, convert to ASCII */)
 {
-  bfs[current].counter = ct;      /* set size of data to write */
-  current = !current;             /* switch to other buffer */
-  if(bfs[current].counter != BF_FREE)     /* if not free */
-    write_behind(test, convert);          /* flush it */
-  bfs[current].counter = BF_ALLOC;        /* mark as allocated */
-  *dpp = &bfs[current].buf.hdr;
-  return ct;                      /* this is a lie of course */
+  struct bf *b;
+
+  bfs[current].counter = BF_FREE; /* free old one */
+  current = !current;             /* "incr" current */
+
+  b = &bfs[current];              /* look at new buffer */
+  if(b->counter == BF_FREE)       /* if empty */
+    read_ahead(test, convert);    /* fill it */
+
+  *dpp = &b->buf.hdr;             /* set caller's ptr */
+  return b->counter;
 }
 
 /*
@@ -482,9 +455,8 @@ static ssize_t write_behind(struct testcase *test, int convert)
     if(prevchar == '\r') {        /* if prev char was cr */
       if(c == '\n')               /* if have cr,lf then just */
         lseek(test->ofile, -1, SEEK_CUR); /* smash lf on top of the cr */
-      else
-        if(c == '\0')             /* if have cr,nul then */
-          goto skipit;            /* just skip over the putc */
+      else if(c == '\0')          /* if have cr,nul then */
+        goto skipit;              /* just skip over the putc */
       /* else just fall through and allow it */
     }
     /* formerly
@@ -497,6 +469,21 @@ skipit:
   return count;
 }
 
+/* Update count associated with the buffer, get new buffer from the queue.
+   Calls write_behind only if next buffer not available.
+ */
+static int writeit(struct testcase *test, struct tftphdr * volatile *dpp,
+                   int ct, int convert)
+{
+  bfs[current].counter = ct;      /* set size of data to write */
+  current = !current;             /* switch to other buffer */
+  if(bfs[current].counter != BF_FREE)     /* if not free */
+    write_behind(test, convert);          /* flush it */
+  bfs[current].counter = BF_ALLOC;        /* mark as allocated */
+  *dpp = &bfs[current].buf.hdr;
+  return ct;                      /* this is a lie of course */
+}
+
 /* When an error has occurred, it is possible that the two sides are out of
  * synch.  Ie: that what I think is the other side's response to packet N is
  * really their response to packet N-1.
@@ -544,6 +531,479 @@ static int synchnet(curl_socket_t f /* socket to flush */)
   return j;
 }
 
+/* Based on the testno, parse the correct server commands. */
+static int tftpd_parse_servercmd(struct testcase *req)
+{
+  FILE *stream;
+  int error;
+
+  stream = test2fopen(req->testno, logdir);
+  if(!stream) {
+    char errbuf[STRERROR_LEN];
+    error = errno;
+    logmsg("fopen() failed with error (%d) %s",
+           error, curlx_strerror(error, errbuf, sizeof(errbuf)));
+    logmsg("  Could not open test file %ld", req->testno);
+    return 1; /* done */
+  }
+  else {
+    char *orgcmd = NULL;
+    char *cmd = NULL;
+    size_t cmdsize = 0;
+    int num = 0;
+
+    /* get the custom server control "commands" */
+    error = getpart(&orgcmd, &cmdsize, "reply", "servercmd", stream);
+    curlx_fclose(stream);
+    if(error) {
+      logmsg("getpart() failed with error (%d)", error);
+      return 1; /* done */
+    }
+
+    cmd = orgcmd;
+    while(cmd && cmdsize) {
+      char *check;
+      if(sscanf(cmd, "writedelay: %d", &num) == 1) {
+        logmsg("instructed to delay %d secs between packets", num);
+        req->writedelay = num;
+      }
+      else {
+        logmsg("Unknown <servercmd> instruction found: %s", cmd);
+      }
+      /* try to deal with CRLF or just LF */
+      check = strchr(cmd, '\r');
+      if(!check)
+        check = strchr(cmd, '\n');
+
+      if(check) {
+        /* get to the letter following the newline */
+        while((*check == '\r') || (*check == '\n'))
+          check++;
+
+        if(!*check)
+          /* if we reached a zero, get out */
+          break;
+        cmd = check;
+      }
+      else
+        break;
+    }
+    free(orgcmd);
+  }
+
+  return 0; /* OK! */
+}
+
+/*
+ * Validate file access.
+ */
+static int validate_access(struct testcase *test,
+                           const char *filename, unsigned short mode)
+{
+  char *ptr;
+
+  logmsg("trying to get file: %s mode %x", filename, mode);
+
+  if(!strncmp("verifiedserver", filename, 14)) {
+    char weare[128];
+    size_t count = snprintf(weare, sizeof(weare), "WE ROOLZ: %ld\r\n",
+                            (long)our_getpid());
+
+    logmsg("Are-we-friendly question received");
+    test->buffer = strdup(weare);
+    test->rptr = test->buffer; /* set read pointer */
+    test->bufsize = count;    /* set total count */
+    test->rcount = count;     /* set data left to read */
+    return 0; /* fine */
+  }
+
+  /* find the last slash */
+  ptr = strrchr(filename, '/');
+
+  if(ptr) {
+    char partbuf[80] = "data";
+    long partno;
+    long testno;
+    const char *pval;
+    curl_off_t num;
+    FILE *stream;
+
+    ptr++; /* skip the slash */
+
+    /* skip all non-numericals following the slash */
+    while(*ptr && !ISDIGIT(*ptr))
+      ptr++;
+
+    /* get the number */
+    pval = ptr;
+    if(!curlx_str_number(&pval, &num, INT_MAX))
+      testno = (long)num;
+    else {
+      logmsg("tftpd: failed to read the test number from '%s'", filename);
+      return TFTP_EACCESS;
+    }
+
+    if(testno > 10000) {
+      partno = testno % 10000;
+      testno /= 10000;
+    }
+    else
+      partno = 0;
+
+    logmsg("requested test number %ld part %ld", testno, partno);
+
+    test->testno = testno;
+
+    (void)tftpd_parse_servercmd(test);
+
+    stream = test2fopen(testno, logdir);
+
+    if(partno)
+      snprintf(partbuf, sizeof(partbuf), "data%ld", partno);
+
+    if(!stream) {
+      char errbuf[STRERROR_LEN];
+      int error = errno;
+      logmsg("fopen() failed with error (%d) %s",
+             error, curlx_strerror(error, errbuf, sizeof(errbuf)));
+      logmsg("Could not open test file for test: %ld", testno);
+      return TFTP_EACCESS;
+    }
+    else {
+      size_t count;
+      int error = getpart(&test->buffer, &count, "reply", partbuf, stream);
+      curlx_fclose(stream);
+      if(error) {
+        logmsg("getpart() failed with error (%d)", error);
+        return TFTP_EACCESS;
+      }
+      if(test->buffer) {
+        test->rptr = test->buffer; /* set read pointer */
+        test->bufsize = count;    /* set total count */
+        test->rcount = count;     /* set data left to read */
+      }
+      else
+        return TFTP_EACCESS;
+    }
+  }
+  else {
+    logmsg("no slash found in path");
+    return TFTP_EACCESS; /* failure */
+  }
+
+  logmsg("file opened and all is good");
+  return 0;
+}
+
+/*
+ * Send the requested file.
+ */
+static void sendtftp(struct testcase *test, const struct formats *pf)
+{
+  int size;
+  ssize_t n;
+  /* These are volatile to live through a siglongjmp */
+  volatile unsigned short sendblock; /* block count */
+  struct tftphdr * volatile sdp = r_init(); /* data buffer */
+  struct tftphdr * const sap = &ackbuf.hdr; /* ack buffer */
+
+  sendblock = 1;
+#if defined(HAVE_ALARM) && defined(SIGALRM)
+  mysignal(SIGALRM, timer);
+#endif
+  do {
+    size = readit(test, (struct tftphdr * volatile *)&sdp, pf->f_convert);
+    if(size < 0) {
+      nak(errno + 100);
+      return;
+    }
+    sdp->th_opcode = htons(opcode_DATA);
+    sdp->th_block = htons(sendblock);
+    timeout = 0;
+#ifdef HAVE_SIGSETJMP
+    (void)sigsetjmp(timeoutbuf, 1);
+#endif
+    if(test->writedelay) {
+      logmsg("Pausing %d seconds before %d bytes", test->writedelay, size);
+      curlx_wait_ms(1000 * test->writedelay);
+    }
+
+send_data:
+    logmsg("write");
+    if(swrite(peer, sdp, size + 4) != size + 4) {
+      logmsg("write: fail");
+      return;
+    }
+    read_ahead(test, pf->f_convert);
+    for(;;) {
+#ifdef HAVE_ALARM
+      alarm(rexmtval);        /* read the ack */
+#endif
+      logmsg("read");
+      n = sread(peer, &ackbuf.storage[0], sizeof(ackbuf.storage));
+      logmsg("read: %zd", n);
+#ifdef HAVE_ALARM
+      alarm(0);
+#endif
+      if(got_exit_signal)
+        return;
+      if(n < 0) {
+        logmsg("read: fail");
+        return;
+      }
+      sap->th_opcode = ntohs(sap->th_opcode);
+      sap->th_block = ntohs(sap->th_block);
+
+      if(sap->th_opcode == opcode_ERROR) {
+        logmsg("got ERROR");
+        return;
+      }
+
+      if(sap->th_opcode == opcode_ACK) {
+        if(sap->th_block == sendblock) {
+          break;
+        }
+        /* Re-synchronize with the other side */
+        (void)synchnet(peer);
+        if(sap->th_block == (sendblock - 1)) {
+          goto send_data;
+        }
+      }
+    }
+    sendblock++;
+  } while(size == SEGSIZE);
+}
+
+/*
+ * Receive a file.
+ */
+static void recvtftp(struct testcase *test, const struct formats *pf)
+{
+  ssize_t n, size;
+  /* These are volatile to live through a siglongjmp */
+  volatile unsigned short recvblock; /* block count */
+  struct tftphdr * volatile rdp;     /* data buffer */
+  struct tftphdr *rap;      /* ack buffer */
+
+  recvblock = 0;
+  rdp = w_init();
+#if defined(HAVE_ALARM) && defined(SIGALRM)
+  mysignal(SIGALRM, timer);
+#endif
+  rap = &ackbuf.hdr;
+  do {
+    timeout = 0;
+    rap->th_opcode = htons(opcode_ACK);
+    rap->th_block = htons(recvblock);
+    recvblock++;
+#ifdef HAVE_SIGSETJMP
+    (void)sigsetjmp(timeoutbuf, 1);
+#endif
+send_ack:
+    logmsg("write");
+    if(swrite(peer, &ackbuf.storage[0], 4) != 4) {
+      logmsg("write: fail");
+      goto abort;
+    }
+    write_behind(test, pf->f_convert);
+    for(;;) {
+#ifdef HAVE_ALARM
+      alarm(rexmtval);
+#endif
+      logmsg("read");
+      n = sread(peer, rdp, PKTSIZE);
+      logmsg("read: %zd", n);
+#ifdef HAVE_ALARM
+      alarm(0);
+#endif
+      if(got_exit_signal)
+        goto abort;
+      if(n < 0) {                        /* really? */
+        logmsg("read: fail");
+        goto abort;
+      }
+      rdp->th_opcode = ntohs(rdp->th_opcode);
+      rdp->th_block = ntohs(rdp->th_block);
+      if(rdp->th_opcode == opcode_ERROR)
+        goto abort;
+      if(rdp->th_opcode == opcode_DATA) {
+        if(rdp->th_block == recvblock) {
+          break;                         /* normal */
+        }
+        /* Re-synchronize with the other side */
+        (void)synchnet(peer);
+        if(rdp->th_block == (recvblock - 1))
+          goto send_ack;                 /* rexmit */
+      }
+    }
+
+    size = writeit(test, &rdp, (int)(n - 4), pf->f_convert);
+    if(size != (n - 4)) {                /* ahem */
+      if(size < 0)
+        nak(errno + 100);
+      else
+        nak(TFTP_ENOSPACE);
+      goto abort;
+    }
+  } while(size == SEGSIZE);
+  write_behind(test, pf->f_convert);
+  /* close the output file as early as possible after upload completion */
+  if(test->ofile > 0) {
+    close(test->ofile);
+    test->ofile = 0;
+  }
+
+  rap->th_opcode = htons(opcode_ACK);  /* send the "final" ack */
+  rap->th_block = htons(recvblock);
+  (void)swrite(peer, &ackbuf.storage[0], 4);
+#if defined(HAVE_ALARM) && defined(SIGALRM)
+  mysignal(SIGALRM, justtimeout);        /* just abort read on timeout */
+  alarm(rexmtval);
+#endif
+  /* normally times out and quits */
+  n = sread(peer, &trsbuf.storage[0], sizeof(trsbuf.storage));
+#ifdef HAVE_ALARM
+  alarm(0);
+#endif
+  if(got_exit_signal)
+    goto abort;
+  if(n >= 4 &&                               /* if read some data */
+     rdp->th_opcode == opcode_DATA &&        /* and got a data block */
+     recvblock == rdp->th_block) {           /* then my last ack was lost */
+    (void)swrite(peer, &ackbuf.storage[0], 4);  /* resend final ack */
+  }
+abort:
+  /* make sure the output file is closed in case of abort */
+  if(test->ofile > 0) {
+    close(test->ofile);
+    test->ofile = 0;
+  }
+  return;
+}
+
+/*
+ * Handle initial connection protocol.
+ */
+static int do_tftp(struct testcase *test, struct tftphdr *tp, ssize_t size)
+{
+  char *cp;
+  int first = 1, ecode;
+  const struct formats *pf;
+  char *filename, *mode = NULL;
+#ifdef USE_WINSOCK
+  DWORD recvtimeout, recvtimeoutbak;
+#endif
+  const char *option = "mode"; /* mode is implicit */
+  int toggle = 1;
+  FILE *server;
+  char dumpfile[256];
+
+  snprintf(dumpfile, sizeof(dumpfile), "%s/%s", logdir, REQUEST_DUMP);
+
+  /* Open request dump file. */
+  server = curlx_fopen(dumpfile, "ab");
+  if(!server) {
+    char errbuf[STRERROR_LEN];
+    int error = errno;
+    logmsg("fopen() failed with error (%d) %s",
+           error, curlx_strerror(error, errbuf, sizeof(errbuf)));
+    logmsg("Error opening file '%s'", dumpfile);
+    return -1;
+  }
+
+  /* store input protocol */
+  fprintf(server, "opcode = %x\n", tp->th_opcode);
+
+  cp = (char *)&tp->th_stuff;
+  filename = cp;
+  do {
+    bool endofit = true;
+    while(cp < &trsbuf.storage[size]) {
+      if(*cp == '\0') {
+        endofit = false;
+        break;
+      }
+      cp++;
+    }
+    if(endofit)
+      /* no more options */
+      break;
+
+    /* before increasing pointer, make sure it is still within the legal
+       space */
+    if((cp + 1) < &trsbuf.storage[size]) {
+      ++cp;
+      if(first) {
+        /* store the mode since we need it later */
+        mode = cp;
+        first = 0;
+      }
+      if(toggle)
+        /* name/value pair: */
+        fprintf(server, "%s = %s\n", option, cp);
+      else {
+        /* store the name pointer */
+        option = cp;
+      }
+      toggle ^= 1;
+    }
+    else
+      /* No more options */
+      break;
+  } while(1);
+
+  if(*cp || !mode) {
+    nak(TFTP_EBADOP);
+    curlx_fclose(server);
+    return 3;
+  }
+
+  /* store input protocol */
+  fprintf(server, "filename = %s\n", filename);
+
+  for(cp = mode; cp && *cp; cp++)
+    if(ISUPPER(*cp))
+      *cp = (char)tolower((int)*cp);
+
+  /* store input protocol */
+  curlx_fclose(server);
+
+  for(pf = formata; pf->f_mode; pf++)
+    if(strcmp(pf->f_mode, mode) == 0)
+      break;
+  if(!pf->f_mode) {
+    nak(TFTP_EBADOP);
+    return 2;
+  }
+  ecode = validate_access(test, filename, tp->th_opcode);
+  if(ecode) {
+    nak(ecode);
+    return 1;
+  }
+
+#ifdef USE_WINSOCK
+  recvtimeout = sizeof(recvtimeoutbak);
+  getsockopt(peer, SOL_SOCKET, SO_RCVTIMEO,
+             (char *)&recvtimeoutbak, (int *)&recvtimeout);
+  recvtimeout = TIMEOUT*1000;
+  setsockopt(peer, SOL_SOCKET, SO_RCVTIMEO,
+             (const char *)&recvtimeout, sizeof(recvtimeout));
+#endif
+
+  if(tp->th_opcode == opcode_WRQ)
+    recvtftp(test, pf);
+  else
+    sendtftp(test, pf);
+
+#ifdef USE_WINSOCK
+  recvtimeout = recvtimeoutbak;
+  setsockopt(peer, SOL_SOCKET, SO_RCVTIMEO,
+             (const char *)&recvtimeout, sizeof(recvtimeout));
+#endif
+
+  return 0;
+}
+
 static int test_tftpd(int argc, char **argv)
 {
   srvr_sockaddr_union_t me;
@@ -648,11 +1108,6 @@ static int test_tftpd(int argc, char **argv)
   snprintf(loglockfile, sizeof(loglockfile), "%s/%s/tftp-%s.lock",
            logdir, SERVERLOGS_LOCKDIR, ipv_inuse);
 
-#ifdef _WIN32
-  if(win32_init())
-    return 2;
-#endif
-
   install_signal_handlers(true);
 
 #ifdef USE_IPV6
@@ -887,507 +1342,3 @@ tftpd_cleanup:
   logmsg("========> tftpd quits");
   return result;
 }
-
-/*
- * Handle initial connection protocol.
- */
-static int do_tftp(struct testcase *test, struct tftphdr *tp, ssize_t size)
-{
-  char *cp;
-  int first = 1, ecode;
-  const struct formats *pf;
-  char *filename, *mode = NULL;
-#ifdef USE_WINSOCK
-  DWORD recvtimeout, recvtimeoutbak;
-#endif
-  const char *option = "mode"; /* mode is implicit */
-  int toggle = 1;
-  FILE *server;
-  char dumpfile[256];
-
-  snprintf(dumpfile, sizeof(dumpfile), "%s/%s", logdir, REQUEST_DUMP);
-
-  /* Open request dump file. */
-  server = curlx_fopen(dumpfile, "ab");
-  if(!server) {
-    char errbuf[STRERROR_LEN];
-    int error = errno;
-    logmsg("fopen() failed with error (%d) %s",
-           error, curlx_strerror(error, errbuf, sizeof(errbuf)));
-    logmsg("Error opening file '%s'", dumpfile);
-    return -1;
-  }
-
-  /* store input protocol */
-  fprintf(server, "opcode = %x\n", tp->th_opcode);
-
-  cp = (char *)&tp->th_stuff;
-  filename = cp;
-  do {
-    bool endofit = true;
-    while(cp < &trsbuf.storage[size]) {
-      if(*cp == '\0') {
-        endofit = false;
-        break;
-      }
-      cp++;
-    }
-    if(endofit)
-      /* no more options */
-      break;
-
-    /* before increasing pointer, make sure it is still within the legal
-       space */
-    if((cp + 1) < &trsbuf.storage[size]) {
-      ++cp;
-      if(first) {
-        /* store the mode since we need it later */
-        mode = cp;
-        first = 0;
-      }
-      if(toggle)
-        /* name/value pair: */
-        fprintf(server, "%s = %s\n", option, cp);
-      else {
-        /* store the name pointer */
-        option = cp;
-      }
-      toggle ^= 1;
-    }
-    else
-      /* No more options */
-      break;
-  } while(1);
-
-  if(*cp || !mode) {
-    nak(TFTP_EBADOP);
-    curlx_fclose(server);
-    return 3;
-  }
-
-  /* store input protocol */
-  fprintf(server, "filename = %s\n", filename);
-
-  for(cp = mode; cp && *cp; cp++)
-    if(ISUPPER(*cp))
-      *cp = (char)tolower((int)*cp);
-
-  /* store input protocol */
-  curlx_fclose(server);
-
-  for(pf = formata; pf->f_mode; pf++)
-    if(strcmp(pf->f_mode, mode) == 0)
-      break;
-  if(!pf->f_mode) {
-    nak(TFTP_EBADOP);
-    return 2;
-  }
-  ecode = validate_access(test, filename, tp->th_opcode);
-  if(ecode) {
-    nak(ecode);
-    return 1;
-  }
-
-#ifdef USE_WINSOCK
-  recvtimeout = sizeof(recvtimeoutbak);
-  getsockopt(peer, SOL_SOCKET, SO_RCVTIMEO,
-             (char *)&recvtimeoutbak, (int *)&recvtimeout);
-  recvtimeout = TIMEOUT*1000;
-  setsockopt(peer, SOL_SOCKET, SO_RCVTIMEO,
-             (const char *)&recvtimeout, sizeof(recvtimeout));
-#endif
-
-  if(tp->th_opcode == opcode_WRQ)
-    recvtftp(test, pf);
-  else
-    sendtftp(test, pf);
-
-#ifdef USE_WINSOCK
-  recvtimeout = recvtimeoutbak;
-  setsockopt(peer, SOL_SOCKET, SO_RCVTIMEO,
-             (const char *)&recvtimeout, sizeof(recvtimeout));
-#endif
-
-  return 0;
-}
-
-/* Based on the testno, parse the correct server commands. */
-static int tftpd_parse_servercmd(struct testcase *req)
-{
-  FILE *stream;
-  int error;
-
-  stream = test2fopen(req->testno, logdir);
-  if(!stream) {
-    char errbuf[STRERROR_LEN];
-    error = errno;
-    logmsg("fopen() failed with error (%d) %s",
-           error, curlx_strerror(error, errbuf, sizeof(errbuf)));
-    logmsg("  Could not open test file %ld", req->testno);
-    return 1; /* done */
-  }
-  else {
-    char *orgcmd = NULL;
-    char *cmd = NULL;
-    size_t cmdsize = 0;
-    int num = 0;
-
-    /* get the custom server control "commands" */
-    error = getpart(&orgcmd, &cmdsize, "reply", "servercmd", stream);
-    curlx_fclose(stream);
-    if(error) {
-      logmsg("getpart() failed with error (%d)", error);
-      return 1; /* done */
-    }
-
-    cmd = orgcmd;
-    while(cmd && cmdsize) {
-      char *check;
-      if(sscanf(cmd, "writedelay: %d", &num) == 1) {
-        logmsg("instructed to delay %d secs between packets", num);
-        req->writedelay = num;
-      }
-      else {
-        logmsg("Unknown <servercmd> instruction found: %s", cmd);
-      }
-      /* try to deal with CRLF or just LF */
-      check = strchr(cmd, '\r');
-      if(!check)
-        check = strchr(cmd, '\n');
-
-      if(check) {
-        /* get to the letter following the newline */
-        while((*check == '\r') || (*check == '\n'))
-          check++;
-
-        if(!*check)
-          /* if we reached a zero, get out */
-          break;
-        cmd = check;
-      }
-      else
-        break;
-    }
-    free(orgcmd);
-  }
-
-  return 0; /* OK! */
-}
-
-/*
- * Validate file access.
- */
-static int validate_access(struct testcase *test,
-                           const char *filename, unsigned short mode)
-{
-  char *ptr;
-
-  logmsg("trying to get file: %s mode %x", filename, mode);
-
-  if(!strncmp("verifiedserver", filename, 14)) {
-    char weare[128];
-    size_t count = snprintf(weare, sizeof(weare), "WE ROOLZ: %ld\r\n",
-                            (long)our_getpid());
-
-    logmsg("Are-we-friendly question received");
-    test->buffer = strdup(weare);
-    test->rptr = test->buffer; /* set read pointer */
-    test->bufsize = count;    /* set total count */
-    test->rcount = count;     /* set data left to read */
-    return 0; /* fine */
-  }
-
-  /* find the last slash */
-  ptr = strrchr(filename, '/');
-
-  if(ptr) {
-    char partbuf[80] = "data";
-    long partno;
-    long testno;
-    const char *pval;
-    curl_off_t num;
-    FILE *stream;
-
-    ptr++; /* skip the slash */
-
-    /* skip all non-numericals following the slash */
-    while(*ptr && !ISDIGIT(*ptr))
-      ptr++;
-
-    /* get the number */
-    pval = ptr;
-    if(!curlx_str_number(&pval, &num, INT_MAX))
-      testno = (long)num;
-    else {
-      logmsg("tftpd: failed to read the test number from '%s'", filename);
-      return TFTP_EACCESS;
-    }
-
-    if(testno > 10000) {
-      partno = testno % 10000;
-      testno /= 10000;
-    }
-    else
-      partno = 0;
-
-    logmsg("requested test number %ld part %ld", testno, partno);
-
-    test->testno = testno;
-
-    (void)tftpd_parse_servercmd(test);
-
-    stream = test2fopen(testno, logdir);
-
-    if(partno)
-      snprintf(partbuf, sizeof(partbuf), "data%ld", partno);
-
-    if(!stream) {
-      char errbuf[STRERROR_LEN];
-      int error = errno;
-      logmsg("fopen() failed with error (%d) %s",
-             error, curlx_strerror(error, errbuf, sizeof(errbuf)));
-      logmsg("Could not open test file for test: %ld", testno);
-      return TFTP_EACCESS;
-    }
-    else {
-      size_t count;
-      int error = getpart(&test->buffer, &count, "reply", partbuf, stream);
-      curlx_fclose(stream);
-      if(error) {
-        logmsg("getpart() failed with error (%d)", error);
-        return TFTP_EACCESS;
-      }
-      if(test->buffer) {
-        test->rptr = test->buffer; /* set read pointer */
-        test->bufsize = count;    /* set total count */
-        test->rcount = count;     /* set data left to read */
-      }
-      else
-        return TFTP_EACCESS;
-    }
-  }
-  else {
-    logmsg("no slash found in path");
-    return TFTP_EACCESS; /* failure */
-  }
-
-  logmsg("file opened and all is good");
-  return 0;
-}
-
-/*
- * Send the requested file.
- */
-static void sendtftp(struct testcase *test, const struct formats *pf)
-{
-  int size;
-  ssize_t n;
-  /* These are volatile to live through a siglongjmp */
-  volatile unsigned short sendblock; /* block count */
-  struct tftphdr * volatile sdp = r_init(); /* data buffer */
-  struct tftphdr * const sap = &ackbuf.hdr; /* ack buffer */
-
-  sendblock = 1;
-#if defined(HAVE_ALARM) && defined(SIGALRM)
-  mysignal(SIGALRM, timer);
-#endif
-  do {
-    size = readit(test, (struct tftphdr * volatile *)&sdp, pf->f_convert);
-    if(size < 0) {
-      nak(errno + 100);
-      return;
-    }
-    sdp->th_opcode = htons(opcode_DATA);
-    sdp->th_block = htons(sendblock);
-    timeout = 0;
-#ifdef HAVE_SIGSETJMP
-    (void)sigsetjmp(timeoutbuf, 1);
-#endif
-    if(test->writedelay) {
-      logmsg("Pausing %d seconds before %d bytes", test->writedelay,
-             size);
-      curlx_wait_ms(1000 * test->writedelay);
-    }
-
-send_data:
-    logmsg("write");
-    if(swrite(peer, sdp, size + 4) != size + 4) {
-      logmsg("write: fail");
-      return;
-    }
-    read_ahead(test, pf->f_convert);
-    for(;;) {
-#ifdef HAVE_ALARM
-      alarm(rexmtval);        /* read the ack */
-#endif
-      logmsg("read");
-      n = sread(peer, &ackbuf.storage[0], sizeof(ackbuf.storage));
-      logmsg("read: %zd", n);
-#ifdef HAVE_ALARM
-      alarm(0);
-#endif
-      if(got_exit_signal)
-        return;
-      if(n < 0) {
-        logmsg("read: fail");
-        return;
-      }
-      sap->th_opcode = ntohs(sap->th_opcode);
-      sap->th_block = ntohs(sap->th_block);
-
-      if(sap->th_opcode == opcode_ERROR) {
-        logmsg("got ERROR");
-        return;
-      }
-
-      if(sap->th_opcode == opcode_ACK) {
-        if(sap->th_block == sendblock) {
-          break;
-        }
-        /* Re-synchronize with the other side */
-        (void)synchnet(peer);
-        if(sap->th_block == (sendblock - 1)) {
-          goto send_data;
-        }
-      }
-    }
-    sendblock++;
-  } while(size == SEGSIZE);
-}
-
-/*
- * Receive a file.
- */
-static void recvtftp(struct testcase *test, const struct formats *pf)
-{
-  ssize_t n, size;
-  /* These are volatile to live through a siglongjmp */
-  volatile unsigned short recvblock; /* block count */
-  struct tftphdr * volatile rdp;     /* data buffer */
-  struct tftphdr *rap;      /* ack buffer */
-
-  recvblock = 0;
-  rdp = w_init();
-#if defined(HAVE_ALARM) && defined(SIGALRM)
-  mysignal(SIGALRM, timer);
-#endif
-  rap = &ackbuf.hdr;
-  do {
-    timeout = 0;
-    rap->th_opcode = htons(opcode_ACK);
-    rap->th_block = htons(recvblock);
-    recvblock++;
-#ifdef HAVE_SIGSETJMP
-    (void)sigsetjmp(timeoutbuf, 1);
-#endif
-send_ack:
-    logmsg("write");
-    if(swrite(peer, &ackbuf.storage[0], 4) != 4) {
-      logmsg("write: fail");
-      goto abort;
-    }
-    write_behind(test, pf->f_convert);
-    for(;;) {
-#ifdef HAVE_ALARM
-      alarm(rexmtval);
-#endif
-      logmsg("read");
-      n = sread(peer, rdp, PKTSIZE);
-      logmsg("read: %zd", n);
-#ifdef HAVE_ALARM
-      alarm(0);
-#endif
-      if(got_exit_signal)
-        goto abort;
-      if(n < 0) {                       /* really? */
-        logmsg("read: fail");
-        goto abort;
-      }
-      rdp->th_opcode = ntohs(rdp->th_opcode);
-      rdp->th_block = ntohs(rdp->th_block);
-      if(rdp->th_opcode == opcode_ERROR)
-        goto abort;
-      if(rdp->th_opcode == opcode_DATA) {
-        if(rdp->th_block == recvblock) {
-          break;                         /* normal */
-        }
-        /* Re-synchronize with the other side */
-        (void)synchnet(peer);
-        if(rdp->th_block == (recvblock-1))
-          goto send_ack;                 /* rexmit */
-      }
-    }
-
-    size = writeit(test, &rdp, (int)(n - 4), pf->f_convert);
-    if(size != (n-4)) {                 /* ahem */
-      if(size < 0)
-        nak(errno + 100);
-      else
-        nak(TFTP_ENOSPACE);
-      goto abort;
-    }
-  } while(size == SEGSIZE);
-  write_behind(test, pf->f_convert);
-  /* close the output file as early as possible after upload completion */
-  if(test->ofile > 0) {
-    close(test->ofile);
-    test->ofile = 0;
-  }
-
-  rap->th_opcode = htons(opcode_ACK);  /* send the "final" ack */
-  rap->th_block = htons(recvblock);
-  (void)swrite(peer, &ackbuf.storage[0], 4);
-#if defined(HAVE_ALARM) && defined(SIGALRM)
-  mysignal(SIGALRM, justtimeout);        /* just abort read on timeout */
-  alarm(rexmtval);
-#endif
-  /* normally times out and quits */
-  n = sread(peer, &trsbuf.storage[0], sizeof(trsbuf.storage));
-#ifdef HAVE_ALARM
-  alarm(0);
-#endif
-  if(got_exit_signal)
-    goto abort;
-  if(n >= 4 &&                               /* if read some data */
-     rdp->th_opcode == opcode_DATA &&        /* and got a data block */
-     recvblock == rdp->th_block) {           /* then my last ack was lost */
-    (void)swrite(peer, &ackbuf.storage[0], 4);  /* resend final ack */
-  }
-abort:
-  /* make sure the output file is closed in case of abort */
-  if(test->ofile > 0) {
-    close(test->ofile);
-    test->ofile = 0;
-  }
-  return;
-}
-
-/*
- * Send a nak packet (error message).  Error code passed in is one of the
- * standard TFTP codes, or a Unix errno offset by 100.
- */
-static void nak(int error)
-{
-  struct tftphdr *tp;
-  int length;
-  struct errmsg *pe;
-
-  tp = &trsbuf.hdr;
-  tp->th_opcode = htons(opcode_ERROR);
-  tp->th_code = htons((unsigned short)error);
-  for(pe = errmsgs; pe->e_code >= 0; pe++)
-    if(pe->e_code == error)
-      break;
-  if(pe->e_code < 0) {
-    curlx_strerror(error - 100, pe->e_msg, sizeof(pe->e_msg));
-    tp->th_code = TFTP_EUNDEF;   /* set 'undef' errorcode */
-  }
-  length = (int)strlen(pe->e_msg);
-
-  /* we use memcpy() instead of strcpy() in order to avoid buffer overflow
-   * report from glibc with FORTIFY_SOURCE */
-  memcpy(tp->th_msg, pe->e_msg, length + 1);
-  length += 5;
-  if(swrite(peer, &trsbuf.storage[0], length) != length)
-    logmsg("nak: fail\n");
-}
